//-----------------------------------------------------------------------------------------------
//    Copyright (c) 
//
//    Name: ald_Parallax_Mapping.fx
//    Author: Gluharev Aleksi Sergeevith (aivle1@yandex.ru)  aka "Aldaron"
//    26.01.2007
//    HTTP://LASTWORLD.NET.RU
//
//    
//-----------------------------------------------------------------------------------------------


float4x4    mat_World_View_Proj; 
float4x4    matWorld;         
float4      View_Pos;        

float4      LightPos; 
float4      LightColor;     
float       LightFalloff;  
       
float       LightAmbient;    
float       LightSpecular;    
float       Attenuation;

float       ParallaxDepth ; 

struct VS_INPUT
{ 
float4 Position   : POSITION;
float3 Normal     : NORMAL;
float2 Tex_UV     : TEXCOORD0;
float3 Tangent    : TANGENT;
float3 Binormal   : BINORMAL;
};

struct VS_OUT
{
float4 Position   : POSITION;
float2 Tex_UV     : TEXCOORD0;
float3 LightDir   : TEXCOORD1;
float3 ViewDir    : TEXCOORD2;
};

VS_OUT vs_main( in VS_INPUT IN ) 
{
VS_OUT OUT;

    OUT.Position = mul(IN.Position, mat_World_View_Proj);  
    OUT.Tex_UV = IN.Tex_UV; //    

    float4 WorldPos = mul(IN.Position, matWorld); 
    OUT.LightDir = LightPos - WorldPos; //  
    
    float3x3 mat_Tang_Binorm_Norm = float3x3(IN.Binormal, IN.Tangent , IN.Normal); 
    
    OUT.LightDir = mul(mat_Tang_Binorm_Norm, OUT.LightDir); //     (    mat_Tang_Binorm_Norm )
    
    float3 ViewDir = View_Pos - WorldPos; //  
    OUT.ViewDir = mul(mat_Tang_Binorm_Norm, ViewDir); //( ) mat_Tang_Binorm_Norm   
     

return OUT;
}

texture baseMap;      

sampler baseSampler = sampler_state
{
Texture = (baseMap);
MIPFILTER = ANISOTROPIC;
MAGFILTER = ANISOTROPIC;
MINFILTER = ANISOTROPIC;
};

texture normalMap;     

sampler normalSampler = sampler_state
{
Texture = (normalMap);
MIPFILTER = ANISOTROPIC;
MAGFILTER = ANISOTROPIC;
MINFILTER = ANISOTROPIC;
};

texture heightMap;    

sampler heightSampler = sampler_state
{
Texture = (heightMap);
MIPFILTER = ANISOTROPIC;
MAGFILTER = ANISOTROPIC;
MINFILTER = ANISOTROPIC;
};


float4 ps_main( in VS_OUT IN ) : COLOR0
{
        
float Height = ParallaxDepth * tex2D(heightSampler, IN.Tex_UV) - 0.02;//  / 
IN.ViewDir = normalize(IN.ViewDir);//    
float2 NewTexCord  = Height * IN.ViewDir + IN.Tex_UV; //     ( )

float4 Color = tex2D(baseSampler, NewTexCord ); //   
float3 Normal = 2.0f * tex2D(normalSampler, NewTexCord ).rgb - 1.0f;

float DotLight = dot( IN.LightDir, IN.LightDir ); // dot product 
IN.LightDir = normalize( IN.LightDir );//     
        
float Attn = min( ((LightFalloff * LightFalloff) / DotLight), Attenuation);// "" (  ) 
float Diffuse = Attn * saturate( dot(Normal, IN.LightDir) );

float3 Ref = normalize(2 * Diffuse * Normal  - IN.LightDir); 
float4 Specular = pow(saturate( dot(Ref, IN.ViewDir) ), LightSpecular);


return Color =  (Diffuse + LightAmbient  + Specular )  * Color * LightColor ; //    
}

technique a_Parallax_Map
{
  pass P0
  {
    vertexshader = compile vs_1_1 vs_main();
    pixelshader = compile ps_2_0 ps_main();
  }
}